package org.mut.ampa.client.screens.login.presenter;

import org.mut.ampa.client.Presenter;
import org.mut.ampa.client.screens.login.LoginServiceAsync;
import org.mut.ampa.client.screens.login.event.ValidateUserEvent;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.model.Usuari;
import org.mut.ampa.shared.model.UsuariRol;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.Widget;

public class LoginPresenter implements Presenter{

	public interface Display  {
	    HasClickHandlers getLoginButton();
	    void setErrorMessage(String msg);
	    String getUserName();
	    String getUserPassword();
	    Widget asWidget();
	  }
	
	private final LoginServiceAsync rpcService;
	private final HandlerManager eventBus;
	private final Display display;
	
	
	public LoginPresenter(LoginServiceAsync rpcService,
							HandlerManager eventBus,
							Display display)
	{
		this.rpcService=rpcService;
		this.eventBus=eventBus;
		this.display=display;
	
		bind();
	}
	@Override
	public void go(HasWidgets container) {
		container.clear();
		container.add(display.asWidget());
		
	}
	
	
	 private void bind() {
		  
		  display.getLoginButton().addClickHandler(new ClickHandler() {
		      public void onClick(ClickEvent event) {
		       	  System.out.println("ASK validate");
		    	  validateUser();
		    	  
		      }
		
		    });

		 }

	 private void validateUser() {
		
		 final LoginCredential credencial = new LoginCredential(display.getUserName(), display.getUserPassword(),UsuariRol.soci);
		 rpcService.validate(credencial, new AsyncCallback<Usuari>() {
			
			@Override
			public void onSuccess(Usuari result) {
				if(result!=null){
					LoginCredential newcred=new LoginCredential(result.getNom()+" "+result.getCognom1()
							                                    , display.getUserPassword(),
							                                    UsuariRol.valueOf(result.getRol()));
					eventBus.fireEvent(new ValidateUserEvent(newcred));
				}else display.setErrorMessage("Usuari invalid");
			}
			
			@Override
			public void onFailure(Throwable caught) {
				display.setErrorMessage("Usuari invalid");
				System.out.println("ASK fail");
			}
		});

		}
}
