package org.mut.ampa.client.screens.manteniment.alumne.presenter;

import java.util.Date;
import java.util.List;

import org.mut.ampa.client.Presenter;
import org.mut.ampa.client.screens.error.event.ErrorAmpaEvent;
import org.mut.ampa.client.screens.manteniment.MantenimentServiceAsync;
import org.mut.ampa.client.screens.manteniment.alumne.event.EditAlumneCancelledEvent;
import org.mut.ampa.client.screens.manteniment.alumne.event.EditAlumneUpdateEvent;
import org.mut.ampa.client.screens.manteniment.tutor.event.EditTutorUpdateEvent;
import org.mut.ampa.client.widget.messagebox.MessageBox;
import org.mut.ampa.client.widget.messagebox.MessageBox.MessageBoxType;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.exceptions.UnAuthorizedException;
import org.mut.ampa.shared.model.Alumne;
import org.mut.ampa.shared.model.AlumneBrief;
import org.mut.ampa.shared.model.CursGrupInfo;
import org.mut.ampa.shared.model.Tutor;
import org.mut.ampa.shared.utils.CCValidator;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasValue;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.Widget;

public class AlumneEditPresenter implements Presenter{

		
	public interface Display {
	    
		HasClickHandlers getSaveButton();
	    HasClickHandlers getCancelButton();
	    
	    HasValue<String> getIdTextBox();
	    HasValue<String> getNomTextBox();
	    HasValue<String> getCognom1TextBox();
	    HasValue<String> getCognom2TextBox();
	    HasValue<Date> getDataNaixementDate();
	    	    
	    HasValue<String> getAdrecaTextBox();
	    HasValue<String> getPoblacioTextBox();
	    ListBox getGrupListBox();
	    ListBox getGenereListBox();
	    HasValue<String> getTSITextBox();
	    
	    HasValue<String> getCCBancTextBox();
	    HasValue<String> getCCOficinaTextBox();
	    HasValue<String> getCCDCTextBox();
	    HasValue<String> getCCNumTextBox();
	    HasClickHandlers getValidateCCButton();
	    ListBox getDretImatgeListBox();
	    
	    HasClickHandlers getAddTutorButton();
	    HasClickHandlers getRemoveTutorButton();
		Tutor getSelectedTutor();  
		void setTutors(List<Tutor> data);
	    
	    HasClickHandlers getAddBrotherButton();
	    HasClickHandlers getRemoveBrotherButton();
	    AlumneBrief getSelectedBrother();  
		void setBrothers(List<AlumneBrief> data);
	    
	    HasValue<String> getObservacionsTextBox();
	    HasValue<String> getAlergiesTextBox();
	    
	    
	    Widget asWidget();
	  }
	
	  private AlumneBrief alumneBrief;
	  

	  private final LoginCredential credencial;
	  private final MantenimentServiceAsync rpcService;
	  private final HandlerManager eventBus;
	  private final Display display;
	  private final boolean modeUpdate;
	  
	  
	  public AlumneEditPresenter(LoginCredential credencial,MantenimentServiceAsync rpcService, HandlerManager eventBus, Display display,AlumneBrief alumne,boolean modeUpdate) 
	  {
	    this.rpcService  = rpcService;
	    this.eventBus    = eventBus;
	    this.display     = display;
	    this.credencial  = credencial;
	    this.alumneBrief = alumne;
	    this.modeUpdate  = modeUpdate;
	    bind();
	    
	    if(alumneBrief!=null) loadFields(); //si no es un registre nou carregar camps.
	    
	  }
	  
	  private void loadFields()
	  {
		loadAlumneInfo();
		loadTutors();
		loadGermans();
		loadGrups();
	  }

	private void loadAlumneInfo() 
	{
		rpcService.getAlumneInfo(credencial,alumneBrief, new AsyncCallback<Alumne>(){

			@Override
			public void onSuccess(Alumne alum) 
			{
			   display.getIdTextBox().setValue(""+alum.getId());
			   display.getNomTextBox().setValue(alum.getNom());
			   display.getCognom1TextBox().setValue(alum.getCognom1());
			   display.getCognom2TextBox().setValue(alum.getCognom2());
			   display.getDataNaixementDate().setValue(alum.getData_naix());
			   display.getAdrecaTextBox().setValue(alum.getAdreca());
			   display.getPoblacioTextBox().setValue(alum.getPoblacio());
			   
			  
			   
			   int numOpcions = display.getGenereListBox().getItemCount();
				for(int i=0;i<numOpcions;i++)
				{
					if(display.getGenereListBox().getItemText(i).equalsIgnoreCase(alum.getGenere()))
					{
						display.getGenereListBox().setSelectedIndex(i);
						break;
					}
				}
				
				display.getTSITextBox().setValue(alum.getTsi());
				display.getCCBancTextBox().setValue(alum.getCc_banc());
				display.getCCOficinaTextBox().setValue(alum.getCc_oficina());
				display.getCCDCTextBox().setValue(alum.getCc_dc());
				display.getCCNumTextBox().setValue(alum.getCc_num());   
			   
				 int numOpcions2 = display.getDretImatgeListBox().getItemCount();
					for(int i=0;i<numOpcions2;i++)
					{
						if(display.getDretImatgeListBox().getItemText(i).equalsIgnoreCase(alum.getUs_imatge()))
						{
							display.getDretImatgeListBox().setSelectedIndex(i);
							break;
						}
						
					}
					
					
			   display.getObservacionsTextBox().setValue(alum.getObservacions());
			   display.getAlergiesTextBox().setValue(alum.getAlergies());
			   
			   
			   
			   
			}
			
			@Override
			public void onFailure(Throwable caught) {
				if(caught instanceof UnAuthorizedException)
					eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			}	
		});
		
	}
	
	private void loadGermans() {
		rpcService.getGermansInfo(credencial,alumneBrief, new AsyncCallback<List<AlumneBrief>>(){
			@Override
			public void onSuccess(List<AlumneBrief> result) {
				display.setBrothers(result);
			}
			@Override
			public void onFailure(Throwable caught) {
				if(caught instanceof UnAuthorizedException)
					eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			}		
		});
		
	}

	private void loadTutors() {
		rpcService.getTutorsInfo(credencial,alumneBrief, new AsyncCallback<List<Tutor>>(){
			@Override
			public void onSuccess(List<Tutor> result) {
				display.setTutors(result);
			}
			@Override
			public void onFailure(Throwable caught) {
				if(caught instanceof UnAuthorizedException)
					eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			}		
		});
		
	}
	
	private void loadGrups(){
		rpcService.getGrupsCurs(credencial,new AsyncCallback<CursGrupInfo>(){

			@Override
			public void onSuccess(CursGrupInfo result) {
				
				int numOpcions2 = display.getGrupListBox().getItemCount();
				for(int i=0;i<numOpcions2;i++)
				{
					if(display.getGrupListBox().getItemText(i).equalsIgnoreCase(result.getDescripcioGrup()))
					{
						display.getGrupListBox().setSelectedIndex(i);
						break;
					}
				}
				
			}
			@Override
			public void onFailure(Throwable caught) {
				if(caught instanceof UnAuthorizedException)
					eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			}		
		
		});
	}

	//Enllaçar als botons els handlers per als click button.
	  public void bind() {
	    
		  this.display.getSaveButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	       if(modeUpdate) doUpdate();
	       else doInsert();
	      }
	    });

	    this.display.getCancelButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	        eventBus.fireEvent(new EditAlumneCancelledEvent());
	      }
	    });
	    
	    this.display.getValidateCCButton().addClickHandler(new ClickHandler() {   
		      public void onClick(ClickEvent event) {
		        validateCC();
		      }
		    });
	  
	  }
	  

	  public void go(final HasWidgets container) {
	    container.clear();
	    container.add(display.asWidget());
	  }

	 
	  
	  
	  private void doInsert() {
	    
		Alumne alumne =  readFields();
	    
		rpcService.insertAlumne(credencial,alumne, new AsyncCallback<AlumneBrief>() {
			    public void onSuccess(AlumneBrief result) {
			      eventBus.fireEvent(new EditAlumneUpdateEvent(result));
			    }
			    public void onFailure(Throwable caught) {
			    	if(caught instanceof UnAuthorizedException)
						eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			    }
			});
		
	  }

	  
		private void doUpdate() {

			Alumne alumne =  readFields();
		    rpcService.updateAlumne(credencial,alumne, new AsyncCallback<AlumneBrief>() {
				    public void onSuccess(AlumneBrief result) {
				      eventBus.fireEvent(new EditAlumneUpdateEvent(result));
				    }
				    public void onFailure(Throwable caught) {
				    	if(caught instanceof UnAuthorizedException)
							eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
				    }
				});
			
		}
		
	private Alumne readFields()
	{
		Alumne registre = new Alumne();
	    
		if (modeUpdate) registre.setId(Integer.valueOf(display.getIdTextBox().getValue()));
		
		registre.setNom(display.getNomTextBox().getValue());
		registre.setCognom1(display.getCognom1TextBox().getValue());
		registre.setCognom2(display.getCognom2TextBox().getValue());
		registre.setData_naix(display.getDataNaixementDate().getValue());
		registre.setAdreca(display.getAdrecaTextBox().getValue());
		registre.setPoblacio(display.getPoblacioTextBox().getValue());
		registre.setGenere(display.getGenereListBox().getItemText(display.getGenereListBox().getSelectedIndex()));
		registre.setTsi(display.getTSITextBox().getValue());
		registre.setCc_banc(display.getCCBancTextBox().getValue());
		registre.setCc_oficina(display.getCCOficinaTextBox().getValue());
		registre.setCc_dc(display.getCCDCTextBox().getValue());
		registre.setCc_num(display.getCCNumTextBox().getValue());
		registre.setUs_imatge(display.getDretImatgeListBox().getValue(display.getDretImatgeListBox().getSelectedIndex()));
		registre.setObservacions(display.getObservacionsTextBox().getValue());
		registre.setAlergies(display.getAlergiesTextBox().getValue());
		
	    return registre;
	}	
	
	
private void validateCC() {
	
	boolean valid = CCValidator.validate(display.getCCBancTextBox().getValue(),
										display.getCCOficinaTextBox().getValue(),
										display.getCCDCTextBox().getValue(),
										display.getCCNumTextBox().getValue());
	
	String msg;
	
	if (valid) msg="El número de compte és correcte";
	else msg="ATENCIÓ !!! \n El número de compte és INCORRECTE";
	
	MessageBox msgBox = new MessageBox(MessageBoxType.INFORMATIU,msg)
	{
		@Override
		protected void onClickYes() {}

		@Override
		protected void onClickNo() {}

		@Override
		protected void onClickOK() {}
	};
	
	msgBox.show();
	
}

}
