package org.mut.ampa.client.screens.manteniment.associacio.presenter;

import org.mut.ampa.client.Presenter;
import org.mut.ampa.client.screens.error.event.ErrorAmpaEvent;
import org.mut.ampa.client.screens.manteniment.MantenimentServiceAsync;
import org.mut.ampa.client.widget.messagebox.MessageBox;
import org.mut.ampa.client.widget.messagebox.MessageBox.MessageBoxType;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.exceptions.UnAuthorizedException;
import org.mut.ampa.shared.model.Associacio;
import org.mut.ampa.shared.utils.CCValidator;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasValue;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.Widget;

public class AssociacioEditPresenter implements Presenter{

	public interface Display {
	    HasClickHandlers getSaveButton();
	    HasClickHandlers getValidateCCButton();
	    HasValue<String> getNomTextBox();
	    HasValue<String> getNifTextBox();
	    HasValue<String> getAdrecaTextBox();
	    HasValue<String> getPoblacioTextBox();
	    HasValue<String> getCCBancTextBox();
	    HasValue<String> getCCOficinaTextBox();
	    HasValue<String> getCCDCTextBox();
	    HasValue<String> getCCNumTextBox();
	    HasValue<String> getSufixTextBox();
	    Widget asWidget();
	  }
	
	private boolean nova_assoc = true;
	
	private final LoginCredential credencial;
	private final MantenimentServiceAsync rpcService;
	private final HandlerManager eventBus;
	private final Display display;
	
	public AssociacioEditPresenter(LoginCredential credencial,MantenimentServiceAsync rpcService, HandlerManager eventBus, Display display) 
	{
		  this.rpcService = rpcService;
		  this.eventBus = eventBus;
		  this.display = display;
		  this.credencial = credencial;

		  bind();
		  
		  
	}
	
	private void bind()
	{
	
		this.display.getSaveButton().addClickHandler(new ClickHandler() {   
		      public void onClick(ClickEvent event) {
		    	  
		    	  Associacio tmp =  readFields();
		    	  
		    	  if(nova_assoc) doInsert(tmp);
		    	  else doUpdate(tmp);
		      }

		
		    });
		
		this.display.getValidateCCButton().addClickHandler(new ClickHandler() {   
		      public void onClick(ClickEvent event) {
		        validateCC();
		      }
		    });
		
	}

	/*
	 * inserir dades associacio
	 */
	private void doInsert(Associacio tmp) {
		rpcService.insertAssociacio(credencial,tmp, new AsyncCallback<Associacio>() {
		    public void onSuccess(Associacio result) {
		    	
		    	MessageBox msgBox = new MessageBox(MessageBoxType.INFORMATIU,"S'han desat les dades correctament"){

					@Override
					protected void onClickYes() {}

					@Override
					protected void onClickNo() {}

					@Override
					protected void onClickOK() {}
		    		
		    	};
		    	
		    	msgBox.show();
		    }
		    public void onFailure(Throwable caught) {
		    	if(caught instanceof UnAuthorizedException)
					eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
		    }
		});
		
	}

	/*
	 * Actualitza dades associacio
	 */
	private void doUpdate(Associacio tmp) {

		rpcService.updateAssociacio(credencial,tmp, new AsyncCallback<Associacio>() {
			    public void onSuccess(Associacio result) {
			    	
			    	MessageBox msgBox = new MessageBox(MessageBoxType.INFORMATIU,"S'han desat les dades correctament"){

						@Override
						protected void onClickYes() {}

						@Override
						protected void onClickNo() {}

						@Override
						protected void onClickOK() {}
			    		
			    	};
			    	
			    	msgBox.show();
			    }
			    public void onFailure(Throwable caught) {
			    	if(caught instanceof UnAuthorizedException)
						eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			    }
			});
		
	}

	@Override
	public void go(HasWidgets container) {
		  container.clear();
		  container.add(display.asWidget());
		  loadAssociacio();
	}
		

	private void validateCC() {
		
		boolean valid = CCValidator.validate(display.getCCBancTextBox().getValue(),
											display.getCCOficinaTextBox().getValue(),
											display.getCCDCTextBox().getValue(),
											display.getCCNumTextBox().getValue());
		
		String msg;
		
		if (valid) msg="El número de compte és correcte";
		else msg="ATENCIÓ !!! \n El número de compte és INCORRECTE";
		
		MessageBox msgBox = new MessageBox(MessageBoxType.INFORMATIU,msg)
		{
			@Override
			protected void onClickYes() {}

			@Override
			protected void onClickNo() {}

			@Override
			protected void onClickOK() {}
		};
		
		msgBox.show();
		
	}
	
	private Associacio readFields()
	{
		Associacio registre = new Associacio();
		
		 registre.setId(1);
		
		registre.setNom(display.getNomTextBox().getValue());
		registre.setNif(display.getNifTextBox().getValue());
		registre.setAdreca(display.getAdrecaTextBox().getValue());
		registre.setPoblacio(display.getPoblacioTextBox().getValue());
		registre.setCc_banc(display.getCCBancTextBox().getValue());
		registre.setCc_oficina(display.getCCOficinaTextBox().getValue());
		registre.setCc_dc(display.getCCDCTextBox().getValue());
		registre.setCc_num(display.getCCNumTextBox().getValue());
		registre.setSufix(display.getSufixTextBox().getValue());
	    
	    return registre;
	}	
	
	private void loadAssociacio() {
		
		rpcService.findAssociacio(credencial,1,new AsyncCallback<Associacio>(){

			@Override
			public void onSuccess(Associacio result) {
				 
				if(result==null) return;
				 nova_assoc=false;
				 
				 display.getNomTextBox().setValue(result.getNom());
				 display.getNifTextBox().setValue(result.getNif());
				 display.getAdrecaTextBox().setValue(result.getAdreca());
				 display.getPoblacioTextBox().setValue(result.getPoblacio());
				 display.getCCBancTextBox().setValue(result.getCc_banc());
				 display.getCCOficinaTextBox().setValue(result.getCc_oficina());
				 display.getCCDCTextBox().setValue(result.getCc_dc());
				 display.getCCNumTextBox().setValue(result.getCc_num());
				 display.getSufixTextBox().setValue(result.getSufix());
			}
			
			@Override
			public void onFailure(Throwable caught) {
				if(caught instanceof UnAuthorizedException)
					eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
				
			}	
		
		
		});
	}
}
