package org.mut.ampa.client.screens.manteniment.contracte.presenter;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.mut.ampa.client.Presenter;
import org.mut.ampa.client.screens.manteniment.MantenimentServiceAsync;
import org.mut.ampa.client.widget.messagebox.MessageBox;
import org.mut.ampa.client.widget.messagebox.MessageBox.MessageBoxType;
import org.mut.ampa.client.widget.selectBox.SelectBox;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.model.AlumneBrief;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.ui.HasValue;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.Widget;

public class ContractarPresenter implements Presenter{

	public interface Display {
	    
	    HasValue<String> getAlumneTextBox();
	    HasClickHandlers getAddAlumneToContractButton();
	    
	    HasClickHandlers getSearchAlumnesButton();
	    HasClickHandlers getSeleccionarAlumne();
	    HasClickHandlers getCancelAddAlumne();
	    ListBox getServeiListBox();
	    HasValue<Date> getStartDate();
	    HasValue<Date> getEndDate();
	    HasClickHandlers getContractarButton();
	    SelectBox<AlumneBrief> searchBox();
	    Widget asWidget();
	  }
	
	private final LoginCredential credencial;
	private final MantenimentServiceAsync rpcService;
	private final HandlerManager eventBus;
	private final Display display;
	
	public ContractarPresenter(LoginCredential credencial,MantenimentServiceAsync rpcService, HandlerManager eventBus, Display display) 
	{
		  this.rpcService = rpcService;
		  this.eventBus = eventBus;
		  this.display = display;
		  this.credencial = credencial;
		  bind();
		  
	}
	
	private void bind()
	{
		//afegir alumne al contracte
		this.display.getAddAlumneToContractButton().addClickHandler(new ClickHandler() {
			
			@Override
			public void onClick(ClickEvent event) {
				display.searchBox().show();
			}
		});
		
		
		//contractar
		this.display.getContractarButton().addClickHandler(new ClickHandler() {
			
			@Override
			public void onClick(ClickEvent event) {
				
				// TODO Auto-generated method stub
				String msg = "L'alumne "+display.getAlumneTextBox().getValue() +" ha contractat el servei: "+display.getServeiListBox().getItemText(display.getServeiListBox().getSelectedIndex());
				//ctrida al servei contractar passant un contracte
				MessageBox msgBox = new MessageBox(MessageBoxType.INFORMATIU,msg)
				{
					@Override
					protected void onClickYes() {}

					@Override
					protected void onClickNo() {}

					@Override
					protected void onClickOK() {
						display.getAlumneTextBox().setValue("");
						display.getServeiListBox().clear();
						}
				};
				
				msgBox.show();
				
			}
		});
		
		//cercar alumne
		this.display.getSearchAlumnesButton().addClickHandler(new ClickHandler() {
			
			@Override
			public void onClick(ClickEvent event) {
				//TODO: MOCK ELIMINAR
				List<AlumneBrief> trobats = new ArrayList<AlumneBrief>();
				AlumneBrief alumn1 = new  AlumneBrief();
				alumn1.setId(2);
				alumn1.setNom("Pol");
				alumn1.setCognom1("Sentí");
				alumn1.setCognom2("Armengol");
				alumn1.setCursDescripcio("P4-A");
				alumn1.setIdsAlumneGerma(new ArrayList<Integer>());
				alumn1.setIdsTutor(new ArrayList<Integer>());
				trobats.add(alumn1);
				
				AlumneBrief alumn2 = new  AlumneBrief();
				alumn2.setId(2);
				alumn2.setNom("Marta");
				alumn2.setCognom1("Ricard");
				alumn2.setCognom2("Armengol");
				alumn2.setCursDescripcio("P3-B");
				alumn2.setIdsAlumneGerma(new ArrayList<Integer>());
				alumn2.setIdsTutor(new ArrayList<Integer>());
				trobats.add(alumn2);
				
				display.searchBox().setTrobats(trobats);
			}
		});
		
		//OK alumne select 
		this.display.getSeleccionarAlumne().addClickHandler(new ClickHandler() {
					
					@Override
					public void onClick(ClickEvent event) {
						int index = display.searchBox().getSelected();
						display.getAlumneTextBox().setValue(display.searchBox().getTrobats().get(index).transform());
						display.searchBox().hide();
					}
				});
		
		//cancelar seleccio alumne 
		this.display.getCancelAddAlumne().addClickHandler(new ClickHandler() {
					
					@Override
					public void onClick(ClickEvent event) {
						display.searchBox().hide();
					}
				});
		
	}

	@Override
	public void go(HasWidgets container) {
		  container.clear();
		  container.add(display.asWidget());
		  initData();
	}

	private void initData() {
		//TODO: ELIMINAR MOCK
		display.getServeiListBox().clear();
		display.getServeiListBox().addItem("");
		display.getServeiListBox().addItem("ACOLLIDA MATINAL 1Q");
		display.getServeiListBox().addItem("ACOLLIDA MATINAL 2Q");
		display.getServeiListBox().addItem("KARATE");
		display.getServeiListBox().addItem("MANUALITATS");
		display.getServeiListBox().addItem("MENJADOR FIXE");
		display.getServeiListBox().addItem("MENJADOR ESPORADIC");
		display.getServeiListBox().addItem("MENJADOR NO SOCI");
		
		
	}
	
}
