package org.mut.ampa.client.screens.manteniment.curs.presenter;

import java.util.Date;

import org.mut.ampa.client.Presenter;
import org.mut.ampa.client.screens.error.event.ErrorAmpaEvent;
import org.mut.ampa.client.screens.manteniment.MantenimentServiceAsync;
import org.mut.ampa.client.screens.manteniment.curs.event.EditCursCancelledEvent;
import org.mut.ampa.client.screens.manteniment.curs.event.EditCursUpdateEvent;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.exceptions.UnAuthorizedException;
import org.mut.ampa.shared.model.Curs;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasValue;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.Widget;

public class CursEditPresenter implements Presenter{

		
	public interface Display {
	    HasClickHandlers getSaveButton();
	    HasClickHandlers getCancelButton();
	    HasValue<String> getIdTextBox();
	    HasValue<String> getDescripcioTextBox();
	    HasValue<Date> getStartDate();
	    HasValue<Date> getEndDate();
	    Widget asWidget();
	  }
	
	  private Curs curs;
	  
	  private final LoginCredential credencial;
	  private final MantenimentServiceAsync rpcService;
	  private final HandlerManager eventBus;
	  private final Display display;
	  private final boolean modeUpdate;
	  
	  
	  public CursEditPresenter(LoginCredential credencial,MantenimentServiceAsync rpcService, HandlerManager eventBus, Display display,Curs curs,boolean modeUpdate) 
	  {
	    this.rpcService = rpcService;
	    this.eventBus = eventBus;
	    this.display = display;
	    this.credencial = credencial;
	    this.curs =curs;
	    this.modeUpdate = modeUpdate;
	    bind();
	    
	    if(curs!=null) loadFields(); //si no es un registre nou carregar camps.
	    
	  }
	  
	  private void loadFields()
	  {
		display.getIdTextBox().setValue(curs.getId().toString());
		display.getDescripcioTextBox().setValue(curs.getDescripcio());
		display.getStartDate().setValue(curs.getInici());
		display.getEndDate().setValue(curs.getFi());
	  }

	//Enllaçar als botons els handlers per als click button.
	  public void bind() {
	    
		  this.display.getSaveButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	       if(modeUpdate) doUpdate();
	       else doInsert();
	      }
	    });

	    this.display.getCancelButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	        eventBus.fireEvent(new EditCursCancelledEvent());
	      }
	    });
	  }

	  public void go(final HasWidgets container) {
	    container.clear();
	    container.add(display.asWidget());
	  }

	 
	  
	  
	  private void doInsert() {
	    
		Curs curs =  readFields();
	    
		rpcService.insertCurs(credencial,curs, new AsyncCallback<Curs>() {
			    public void onSuccess(Curs result) {
			      eventBus.fireEvent(new EditCursUpdateEvent(result));
			    }
			    public void onFailure(Throwable caught) {
			    	if(caught instanceof UnAuthorizedException)
						eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			    }
			});
		
	  }

	  
		private void doUpdate() {

			Curs curs =  readFields();
		    rpcService.updateCurs(credencial,curs, new AsyncCallback<Curs>() {
				    public void onSuccess(Curs result) {
				      eventBus.fireEvent(new EditCursUpdateEvent(result));
				    }
				    public void onFailure(Throwable caught) {
				    	if(caught instanceof UnAuthorizedException)
							eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
				    }
				});
			
		}
		
	private Curs readFields()
	{
		Curs registre = new Curs();
	    
		if (modeUpdate) registre.setId(Integer.valueOf(display.getIdTextBox().getValue()));
		
		registre.setDescripcio(display.getDescripcioTextBox().getValue());
		registre.setInici(display.getStartDate().getValue());
		registre.setFi(display.getEndDate().getValue());
	    
	    return registre;
	}	
}
