package org.mut.ampa.client.screens.manteniment.curs.presenter;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.mut.ampa.client.Presenter;
import org.mut.ampa.client.screens.error.event.ErrorAmpaEvent;
import org.mut.ampa.client.screens.manteniment.MantenimentServiceAsync;
import org.mut.ampa.client.screens.manteniment.curs.event.EditCursEvent;
import org.mut.ampa.client.widget.filterbox.FilterBox;
import org.mut.ampa.client.widget.messagebox.MessageBox;
import org.mut.ampa.client.widget.messagebox.MessageBox.MessageBoxType;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.exceptions.UnAuthorizedException;
import org.mut.ampa.shared.filter.Filter;
import org.mut.ampa.shared.model.Curs;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.ToggleButton;
import com.google.gwt.user.client.ui.Widget;

/**
 * Presentador per a la Vista en Grid de l'Entitat Curs
 * Implementa tota la logica per al manteniment.
 * 
 * @author Mario Unio UOC 2012
 *
 */
public class CursListPresenter implements Presenter {

	// Interficie utilitzada pel Presenter per accedir a la Vista
	public interface Display 
	{
		HasClickHandlers getAddButton();
		HasClickHandlers getDeleteButton();
		HasClickHandlers getFilterButton();
		HasClickHandlers getEditButton();
		Curs getSelectedEntity();  
		void setData(List<Curs> data);
		Map<Integer,String> getFilterFields();
		Widget asWidget();
	}

	private final MantenimentServiceAsync rpcService;
	private final HandlerManager eventBus;
	private final Display display;
	private List<Curs> entities;
	private LoginCredential credential;

	public CursListPresenter(LoginCredential credential,
								MantenimentServiceAsync rpcService,
								HandlerManager eventBus, 
								Display display) 
	{
		this.rpcService = rpcService;
		this.eventBus = eventBus;
		this.display = display;
		this.credential = credential;
		bind();
	}

	private void bind() {
		display.getAddButton().addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				eventBus.fireEvent(new EditCursEvent(null,false));
			}
		});

		display.getDeleteButton().addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				deleteCurs();
			}
		});
		
		display.getFilterButton().addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				if(((ToggleButton)display.getFilterButton()).isDown())
				{
					searchCurs();
				}else{
					//Desmarquem boto cercar
					((ToggleButton)display.getFilterButton()).setValue(false, false);
					//recuperem els resultats sense cerca
					fetchCurs();
				} 
			}
		});
		
		display.getEditButton().addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				eventBus.fireEvent(new EditCursEvent(display.getSelectedEntity(),true));
			}
		});
		
		
			}

	@Override
	public void go(HasWidgets container) {
		container.clear();
		container.add(display.asWidget());
		fetchCurs();

	}

	private void deleteCurs() {
					
			//no fer res en el cas de cap registre seleccionat
			if(display.getSelectedEntity()==null) return;
			
			MessageBox msgBox = new MessageBox(MessageBoxType.PREGUNTA,"Vols esborrar el registre seleccionat")
			{

				@Override
				protected void onClickYes() {
						
					rpcService.deleteCurs(credential,display.getSelectedEntity().getId(),new AsyncCallback<List<Curs>>() {
						
						public void onSuccess(List<Curs> result) {
							
							display.setData(result);
						}

						public void onFailure(Throwable caught) {
							if(caught instanceof UnAuthorizedException)
								eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
						}
					});

					
				}

				@Override
				protected void onClickNo() {}

				@Override
				protected void onClickOK() {}
			
			};
			
			msgBox.show();
				
					

	}
	
	private void searchCurs() {
		 FilterBox filter = new FilterBox("Filtrar",display.getFilterFields()) {
			
			@Override
			protected void onClickAcceptar()
			{
				Filter filterValues = getValues();
				
				rpcService.findCurs(credential, 
									filterValues,
									  new AsyncCallback<List<Curs>>()
									  		{
												@Override
												public void onSuccess(List<Curs> result) 
												{
													entities = result;
													List<Curs> data = new ArrayList<Curs>();

													for (int i = 0; i < result.size(); ++i) {
														data.add(entities.get(i));
													}

													display.setData(data);
						
												}
												@Override
												public void onFailure(Throwable caught) 
												{
													if(caught instanceof UnAuthorizedException)
														eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
						
												}
									  		} );
			}
		};
		filter.show();
	}

	private void fetchCurs() {
		
			rpcService.getCursos(credential, 0, 5, new AsyncCallback<List<Curs>>() {
				public void onSuccess(List<Curs> result) {
					entities = result;
					List<Curs> data = new ArrayList<Curs>();

					for (int i = 0; i < result.size(); ++i) {
						data.add(entities.get(i));
					}

					display.setData(data);
				}

				public void onFailure(Throwable caught) {
					if(caught instanceof UnAuthorizedException)
					eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
				}
			});

	}

}
