package org.mut.ampa.client.screens.manteniment.grup.presenter;

import java.util.HashMap;
import java.util.List;

import org.mut.ampa.client.Presenter;
import org.mut.ampa.client.screens.error.event.ErrorAmpaEvent;
import org.mut.ampa.client.screens.manteniment.MantenimentServiceAsync;
import org.mut.ampa.client.screens.manteniment.grup.event.EditGrupCancelledEvent;
import org.mut.ampa.client.screens.manteniment.grup.event.EditGrupUpdateEvent;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.exceptions.UnAuthorizedException;
import org.mut.ampa.shared.model.Curs;
import org.mut.ampa.shared.model.CursGrupInfo;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasValue;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.Widget;

public class GrupEditPresenter implements Presenter{

		
	public interface Display {
	    HasClickHandlers getSaveButton();
	    HasClickHandlers getCancelButton();
	    HasValue<String> getIdGrupTextBox();
	    HasValue<String> getGrupDescripcioTextBox();
	    ListBox getCursListBox();
	    Widget asWidget();
	  }
	
	  private CursGrupInfo grup;
	  private HashMap<String,Curs> cursos = new HashMap<String,Curs>();//Key descripcio curs
	  
	  private final LoginCredential credencial;
	  private final MantenimentServiceAsync rpcService;
	  private final HandlerManager eventBus;
	  private final Display display;
	  private final boolean modeUpdate;
	  
	  
	  public GrupEditPresenter(LoginCredential credencial,MantenimentServiceAsync rpcService, HandlerManager eventBus, Display display,CursGrupInfo grup,boolean modeUpdate) 
	  {
	    this.rpcService = rpcService;
	    this.eventBus = eventBus;
	    this.display = display;
	    this.credencial = credencial;
	    this.grup =grup;
	    this.modeUpdate = modeUpdate;
	    bind();
	    
	    //Recuperem els cursos 
	    initCursos();
	    
	    //si no es un registre nou carregar camps.
	    if(grup!=null) loadFields(); 
	    
	  }
	  
	  private void initCursos()
	  {
		  cursos.clear();
		  rpcService.getCursos(credencial, 0, 5, new AsyncCallback<List<Curs>>() {
				public void onSuccess(List<Curs> result) 
							{

							for (int i = 0; i < result.size(); ++i) 
								{
									cursos.put(result.get(i).getDescripcio(), result.get(i));
									display.getCursListBox().addItem(result.get(i).getDescripcio());
								}
							}

				public void onFailure(Throwable caught) {
					if(caught instanceof UnAuthorizedException)
					eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
				}
			});
	  }

	private void loadFields() {
		display.getIdGrupTextBox().setValue(grup.getIdGrup().toString());
		display.getGrupDescripcioTextBox().setValue(grup.getDescripcioGrup());
		
		
		int numOpcions = display.getCursListBox().getItemCount();
		for(int i=0;i<numOpcions;i++)
		{
			if(display.getCursListBox().getItemText(i).equalsIgnoreCase(grup.getDescripcioCurs()))
			{
				display.getCursListBox().setSelectedIndex(i);
				break;
			}
		}
		
	}

	//Enllaçar als botons els handlers per als click button.
	  public void bind() {
	    
		  this.display.getSaveButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	       if(modeUpdate) doUpdate();
	       else doInsert();
	      }
	    });

	    this.display.getCancelButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	        eventBus.fireEvent(new EditGrupCancelledEvent());
	      }
	    });
	  }

	  public void go(final HasWidgets container) {
	    container.clear();
	    container.add(display.asWidget());
	  }

	 
	  
	  
	private void doInsert() {
	    
		CursGrupInfo grup =  readFields();
	    
		rpcService.insertGrup(credencial,grup, new AsyncCallback<CursGrupInfo>() {
			    public void onSuccess(CursGrupInfo result) {
			      eventBus.fireEvent(new EditGrupUpdateEvent(result));
			    }
			    public void onFailure(Throwable caught) {
			    	if(caught instanceof UnAuthorizedException)
						eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			    }
			});
		
	  }

	  
	private void doUpdate() {

			CursGrupInfo grup =  readFields();
		    rpcService.updateGrup(credencial,grup, new AsyncCallback<CursGrupInfo>() {
				    public void onSuccess(CursGrupInfo result) {
				      eventBus.fireEvent(new EditGrupUpdateEvent(result));
				    }
				    public void onFailure(Throwable caught) {
				    	if(caught instanceof UnAuthorizedException)
							eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
				    }
				});
			
		}
		
	private CursGrupInfo readFields()
	{
		CursGrupInfo registre = new CursGrupInfo();
	    
		if (modeUpdate) registre.setIdGrup(Integer.valueOf(display.getIdGrupTextBox().getValue()));
		
		registre.setDescripcioGrup(display.getGrupDescripcioTextBox().getValue());
		
		String selectDesc =display.getCursListBox().getValue(display.getCursListBox().getSelectedIndex());
		registre.setDescripcioCurs(selectDesc);
		
		int idCurs = cursos.get(selectDesc).getId();
		registre.setIdCurs(idCurs);

		return registre;
	}	
}
