package org.mut.ampa.client.screens.manteniment.tutor.presenter;

import org.mut.ampa.client.Presenter;
import org.mut.ampa.client.screens.error.event.ErrorAmpaEvent;
import org.mut.ampa.client.screens.manteniment.MantenimentServiceAsync;
import org.mut.ampa.client.screens.manteniment.tutor.event.EditTutorCancelledEvent;
import org.mut.ampa.client.screens.manteniment.tutor.event.EditTutorUpdateEvent;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.exceptions.UnAuthorizedException;
import org.mut.ampa.shared.model.Tutor;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasValue;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.Widget;

public class TutorEditPresenter implements Presenter{

		
	public interface Display {
	    HasClickHandlers getSaveButton();
	    HasClickHandlers getCancelButton();
	    HasValue<String> getIdTextBox();
	    HasValue<String> getNomTextBox();
	    HasValue<String> getCognom1TextBox();
	    HasValue<String> getCognom2TextBox();
	    HasValue<String> getTelefon1TextBox();
	    HasValue<String> getTelefon2TextBox();
	    HasValue<String> getEmailTextBox();
	    Widget asWidget();
	  }
	
	  private Tutor tutor;

	  private final LoginCredential credencial;
	  private final MantenimentServiceAsync rpcService;
	  private final HandlerManager eventBus;
	  private final Display display;
	  private final boolean modeUpdate;
	  
	  
	  public TutorEditPresenter(LoginCredential credencial,MantenimentServiceAsync rpcService, HandlerManager eventBus, Display display,Tutor tutor,boolean modeUpdate) 
	  {
	    this.rpcService = rpcService;
	    this.eventBus = eventBus;
	    this.display = display;
	    this.credencial = credencial;
	    this.tutor =tutor;
	    this.modeUpdate = modeUpdate;
	    bind();
	    
	    if(tutor!=null) loadFields(); //si no es un registre nou carregar camps.
	    
	  }
	  
	  private void loadFields()
	  {
		display.getIdTextBox().setValue(tutor.getId().toString());
		display.getNomTextBox().setValue(tutor.getNom().toString());
		display.getCognom1TextBox().setValue(tutor.getCognom1().toString());
		display.getCognom2TextBox().setValue(tutor.getCognom2().toString());
		display.getTelefon1TextBox().setValue(tutor.getTelefon1().toString());
		display.getTelefon2TextBox().setValue(tutor.getTelefon2().toString());
		display.getEmailTextBox().setValue(tutor.getEmail().toString());
	  }

	//Enllaçar als botons els handlers per als click button.
	  public void bind() {
	    
		  this.display.getSaveButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	       if(modeUpdate) doUpdate();
	       else doInsert();
	      }
	    });

	    this.display.getCancelButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	        eventBus.fireEvent(new EditTutorCancelledEvent());
	      }
	    });
	  }

	  public void go(final HasWidgets container) {
	    container.clear();
	    container.add(display.asWidget());
	  }

	 
	  
	  
	  private void doInsert() {
	    
		Tutor tutor =  readFields();
	    
		rpcService.insertTutor(credencial,tutor, new AsyncCallback<Tutor>() {
			    public void onSuccess(Tutor result) {
			      eventBus.fireEvent(new EditTutorUpdateEvent(result));
			    }
			    public void onFailure(Throwable caught) {
			    	if(caught instanceof UnAuthorizedException)
						eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			    }
			});
		
	  }

	  
		private void doUpdate() {

			Tutor tutor =  readFields();
		    rpcService.updateTutor(credencial,tutor, new AsyncCallback<Tutor>() {
				    public void onSuccess(Tutor result) {
				      eventBus.fireEvent(new EditTutorUpdateEvent(result));
				    }
				    public void onFailure(Throwable caught) {
				    	if(caught instanceof UnAuthorizedException)
							eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
				    }
				});
			
		}
		
	private Tutor readFields()
	{
		Tutor registre = new Tutor();
	    
		if (modeUpdate) registre.setId(Integer.valueOf(display.getIdTextBox().getValue()));
		
		registre.setNom(display.getNomTextBox().getValue());
		registre.setCognom1(display.getCognom1TextBox().getValue());
		registre.setCognom2(display.getCognom2TextBox().getValue());
		registre.setTelefon1(display.getTelefon1TextBox().getValue());
		registre.setTelefon2(display.getTelefon2TextBox().getValue());
		registre.setEmail(display.getEmailTextBox().getValue());
		
	    return registre;
	}	
}
