package org.mut.ampa.client.screens.manteniment.usuari.presenter;

import org.mut.ampa.client.Presenter;
import org.mut.ampa.client.screens.error.event.ErrorAmpaEvent;
import org.mut.ampa.client.screens.manteniment.MantenimentServiceAsync;
import org.mut.ampa.client.screens.manteniment.usuari.event.EditUsuariCancelledEvent;
import org.mut.ampa.client.screens.manteniment.usuari.event.EditUsuariUpdateEvent;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.exceptions.UnAuthorizedException;
import org.mut.ampa.shared.model.Usuari;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HasValue;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.Widget;

public class UsuariEditPresenter implements Presenter{

		
	public interface Display {
	    HasClickHandlers getSaveButton();
	    HasClickHandlers getCancelButton();
	    HasValue<String> getIdTextBox();
	    HasValue<String> getNomUsuariTextBox();
	    HasValue<String> getPasswordTextBox();
	    HasValue<String> getNomTextBox();
	    HasValue<String> getCognom1TextBox();
	    HasValue<String> getCognom2TextBox();
	    ListBox getRolListBox();
	    Widget asWidget();
	  }
	
	  private Usuari user;
	  
	  private final LoginCredential credencial;
	  private final MantenimentServiceAsync rpcService;
	  private final HandlerManager eventBus;
	  private final Display display;
	  private final boolean modeUpdate;
	  
	  
	  public UsuariEditPresenter(LoginCredential credencial,MantenimentServiceAsync rpcService, HandlerManager eventBus, Display display,Usuari user,boolean modeUpdate) 
	  {
	    this.rpcService = rpcService;
	    this.eventBus = eventBus;
	    this.display = display;
	    this.credencial = credencial;
	    this.user =user;
	    this.modeUpdate = modeUpdate;
	    bind();
	    
	    if(user!=null) loadFields(); //si no es un registre nou carregar camps.
	    
	  }
	  
	  private void loadFields() {
		display.getIdTextBox().setValue(user.getId().toString());
		display.getNomUsuariTextBox().setValue(user.getNomUsuari());
		display.getPasswordTextBox().setValue(user.getPasswd());
		display.getNomTextBox().setValue(user.getNom());
		display.getCognom1TextBox().setValue(user.getCognom1());
		display.getCognom2TextBox().setValue(user.getCognom2());
		
		int numOpcions = display.getRolListBox().getItemCount();
		for(int i=0;i<numOpcions;i++)
		{
			if(display.getRolListBox().getItemText(i).equalsIgnoreCase(user.getRol()))
			{
				display.getRolListBox().setSelectedIndex(i);
				break;
			}
		}
		
	}

	//Enllaçar als botons els handlers per als click button.
	  public void bind() {
	    
		  this.display.getSaveButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	       if(modeUpdate) doUpdate();
	       else doInsert();
	      }
	    });

	    this.display.getCancelButton().addClickHandler(new ClickHandler() {   
	      public void onClick(ClickEvent event) {
	        eventBus.fireEvent(new EditUsuariCancelledEvent());
	      }
	    });
	  }

	  public void go(final HasWidgets container) {
	    container.clear();
	    container.add(display.asWidget());
	  }

	 
	  
	  
	  private void doInsert() {
	    
		Usuari user =  readFields();
	    
		rpcService.insertUsuari(credencial,user, new AsyncCallback<Usuari>() {
			    public void onSuccess(Usuari result) {
			      eventBus.fireEvent(new EditUsuariUpdateEvent(result));
			    }
			    public void onFailure(Throwable caught) {
			    	if(caught instanceof UnAuthorizedException)
						eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
			    }
			});
		
	  }

	  
		private void doUpdate() {

			Usuari user =  readFields();
		    rpcService.updateUsuari(credencial,user, new AsyncCallback<Usuari>() {
				    public void onSuccess(Usuari result) {
				      eventBus.fireEvent(new EditUsuariUpdateEvent(result));
				    }
				    public void onFailure(Throwable caught) {
				    	if(caught instanceof UnAuthorizedException)
							eventBus.fireEvent(new ErrorAmpaEvent(((UnAuthorizedException)caught).getErrorInfo()));
				    }
				});
			
		}
		
	private Usuari readFields()
	{
		Usuari registre = new Usuari();
	    
		if (modeUpdate) registre.setId(Integer.valueOf(display.getIdTextBox().getValue()));
		
		registre.setNomUsuari(display.getNomUsuariTextBox().getValue());
		registre.setPasswd(display.getPasswordTextBox().getValue());
		registre.setNom(display.getNomTextBox().getValue());
		registre.setCognom1(display.getCognom1TextBox().getValue());
		registre.setCognom2(display.getCognom2TextBox().getValue());
		registre.setRol(display.getRolListBox().getValue(display.getRolListBox().getSelectedIndex()));
	    
	    return registre;
	}	
}
