package org.mut.ampa.client.widget.datagrid;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HasHandlers;
import com.google.gwt.user.cellview.client.ColumnSortEvent.ListHandler;
import com.google.gwt.user.cellview.client.DataGrid;
import com.google.gwt.user.cellview.client.SimplePager;
import com.google.gwt.user.cellview.client.SimplePager.TextLocation;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.PushButton;
import com.google.gwt.user.client.ui.ToggleButton;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.view.client.ListDataProvider;
import com.google.gwt.view.client.NoSelectionModel;
import com.google.gwt.view.client.SelectionChangeEvent;

/**
 * Control que representa un grid per a dades en forma de llista, 
 * amb una botonera per a les operacions basiques.
 * @author Mario Unio UOC 2012
 *
 * @param <TipusFila>
 */
public abstract class DataGridPaginat<TipusFila> extends Composite{

	private final DataGridResources resources =  GWT.create(DataGridResources.class);
	
	
	protected boolean visible[];
	
	final private VerticalPanel root = new VerticalPanel();
	
	private TipusFila currentFila;
	public TipusFila getCurrentFila() {
		return currentFila;
	}

	private final HorizontalPanel barraBotons = new HorizontalPanel();
	private final PushButton addButton; 
		protected HasClickHandlers getParentAddButton(){return addButton;}
	private final PushButton deleteButton;
		protected HasClickHandlers getParentDeleteButton(){return deleteButton;}
	
	private final PushButton editButton ;
		protected HasClickHandlers getParentEditButton(){return editButton;}
		
	private final ToggleButton filtreButton ;
		protected HasClickHandlers getParentSearchButton(){return filtreButton;}
		
		
	private DataGrid<TipusFila> dataGrid ;	
		protected HasHandlers getParentList() {return dataGrid;}
	final private SimplePager pager;
	
	private ListDataProvider<TipusFila> dataProvider;
	private final NoSelectionModel<TipusFila> selectionModel = new NoSelectionModel<TipusFila>();

	private String height;


	public DataGridPaginat(String title,int pageSize,boolean visible[],int h,int w) {
		
		resources.style().ensureInjected();
		
		addButton    = new PushButton(new Image(resources.addImage()));
		addButton.setStyleName(resources.style().elementButtonBar());
		editButton = new PushButton(new Image(resources.pencilImage()));
		editButton.setStyleName(resources.style().elementButtonBar());
		deleteButton = new PushButton(new Image(resources.crossImage()));
		deleteButton.setStyleName(resources.style().elementButtonBar());
		filtreButton = new ToggleButton(new Image(resources.filterImage()),new Image(resources.unfilterImage()));
		filtreButton.setStyleName(resources.style().elementButtonBar());
		
		
		this.visible=visible;
		//Iniciar Widget contenidor (arrel del control)
		initWidget(root);
		root.setStyleName(resources.style().listWindow());
		dataGrid =new DataGrid<TipusFila>(pageSize);
		dataGrid.setHeight(h+"px");
		dataGrid.setWidth(w+"px");
		dataGrid.setVisible(true);
		dataGrid.setEmptyTableWidget(new HTML("Sense Dades"));
		
		
		SimplePager.Resources pagerResources = GWT.create(SimplePager.Resources.class);
        pager = new SimplePager(TextLocation.CENTER, pagerResources, false, 0,true);
        pager.setDisplay(dataGrid);
		
		pager.setWidth("100%");
		pager.setVisible(true);
		
		
		dataProvider = new ListDataProvider<TipusFila>();
		dataProvider.setList(new ArrayList<TipusFila>());
		
		ListHandler<TipusFila> sortHandler = new ListHandler<TipusFila>(dataProvider.getList());
		initColumnesTaula(dataGrid, sortHandler);
		dataProvider.addDataDisplay(dataGrid);
		dataGrid.addColumnSortHandler(sortHandler);
        
		//Definim el model de seleccio que sera de 1 fila
		dataGrid.setSelectionModel(selectionModel);		
		selectionModel.addSelectionChangeHandler(new SelectionChangeEvent.Handler() {
			
			public void onSelectionChange(SelectionChangeEvent event) {
				currentFila = selectionModel.getLastSelectedObject();
				
			}
		});
		
		//Afegit titol
		Label screenTitle = new Label(title);
		screenTitle.setStyleName(resources.style().titleBar());
		//Afegim la taula i el paginador
		barraBotons.add(addButton);
		barraBotons.add(editButton);
		barraBotons.add(deleteButton);
		barraBotons.add(filtreButton);
		
		barraBotons.setStyleName(resources.style().buttonBar());
		
		root.add(screenTitle);
		root.add(barraBotons);
		root.add(dataGrid);
		root.add(pager);
		
		
	}

	/**
	 * Metode per afegir una columna al Grid
	 * @param dataGrid	  Datagrid.
	 * @param sortHandler Handler per a la ordenació.
	 */
	public abstract void initColumnesTaula(DataGrid<TipusFila> dataGrid,ListHandler<TipusFila> sortHandler);
	
    public List<TipusFila> getDataList() {
        	return dataProvider.getList();
    }
 
    public void setDataList(List<TipusFila> dataList) 
    {
    	 dataProvider.setList(dataList);
    	 dataProvider.refresh();
    }
 
    public ListDataProvider<TipusFila> getDataProvider() {
        return dataProvider;
    }
 
    public void setDataProvider(ListDataProvider<TipusFila> dataProvider) {
        this.dataProvider = dataProvider;
    }

    public String getHeight() {
    	        return height;
    	    }
    	 
    public void setHeight(String height) {
    	        this.height = height;
    	        dataGrid.setHeight(height);
    	    }

	
	
}
