package org.mut.ampa.client.widget.messagebox;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.LayoutPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * Classe que permet mostrar missatges emergents.
 * Els tipus són :
 * 	Informatiu  : Mostra una informació a l'usuari
 *  Pregunta    : Realitza una consulta a l'usuari que la resposta pot ser o Si o No.
 *  Error       : Informa que s'ha produït un error i mostra l'informació d'aquest.
 * @author Mario Unio UOC 2012
 *
 */
public  abstract class MessageBox extends DialogBox{

	private final MessageBoxResources resources =  GWT.create(MessageBoxResources.class);
	
	public enum MessageBoxType{ INFORMATIU, PREGUNTA, ERROR};
	
	
	protected abstract void onClickYes();
	protected abstract void onClickNo();
	protected abstract void onClickOK();
	
	private final VerticalPanel root = new VerticalPanel();
	
	public MessageBox(MessageBoxType type,String message) {
		super(false,true);
		
		
		
		
		Label missatge = new Label(message);
		missatge.setHeight("40px");
		root.getElement().getStyle().setPadding(20, Unit.PX);
		root.add(missatge);
		missatge.setStyleName(resources.style().message());
		
		LayoutPanel barraBotons = new LayoutPanel();
		barraBotons.setWidth("100%");
		barraBotons.setHeight("100%");
		root.add(barraBotons);
		
		switch(type)
		{
		case INFORMATIU:
			Button infoOk = new Button("OK");
			infoOk.addClickHandler(new ClickHandler() {
				
				@Override
				public void onClick(ClickEvent event) {
					onClickOK();  
					MessageBox.this.hide();
				}
			});
			barraBotons.add(infoOk);	
			infoOk.getElement().getStyle().setMarginBottom(5, Unit.PX);
			barraBotons.setWidgetTopHeight(infoOk, 0, Unit.PX, 30, Unit.PX);
			barraBotons.setWidgetLeftWidth(infoOk, 35, Unit.PCT, 100, Unit.PX);
			break;
		case PREGUNTA:
			Button preguntaSi = new Button("Si");
			preguntaSi.addClickHandler(new ClickHandler() {
				
				@Override
				public void onClick(ClickEvent event) {
					onClickYes();  
					MessageBox.this.hide();
				}
			});
			barraBotons.add(preguntaSi);	
			preguntaSi.getElement().getStyle().setMarginBottom(5, Unit.PX);
			barraBotons.setWidgetTopHeight(preguntaSi, 0, Unit.PX, 30, Unit.PX);
			barraBotons.setWidgetLeftWidth(preguntaSi, 0, Unit.PX, 100, Unit.PX);
			
			Button preguntaNo = new Button("No");
			preguntaNo.addClickHandler(new ClickHandler() {
				
				@Override
				public void onClick(ClickEvent event) {
					onClickNo();  
					MessageBox.this.hide();
				}
			});
			barraBotons.add(preguntaNo);	
			preguntaNo.getElement().getStyle().setMarginBottom(5, Unit.PX);
			barraBotons.setWidgetTopHeight(preguntaNo, 0, Unit.PX, 30, Unit.PX);
			barraBotons.setWidgetRightWidth(preguntaNo, 0, Unit.PX, 100, Unit.PX);
			break;
		case ERROR:
			Button errorOk = new Button("OK");
			errorOk.addClickHandler(new ClickHandler() {
				
				@Override
				public void onClick(ClickEvent event) {
					onClickOK();  
					MessageBox.this.hide();
				}
			});
			barraBotons.add(errorOk);
			errorOk.getElement().getStyle().setMarginBottom(5, Unit.PX);
			barraBotons.setWidgetTopHeight(errorOk, 0, Unit.PX, 30, Unit.PX);
			barraBotons.setWidgetRightWidth(errorOk, 50, Unit.PCT, 100, Unit.PX);
			break;
		default:
			break;
		}
		//afegim el contingut al DialogBox
		setWidget(root);
		
		MessageBox.this.setGlassEnabled(true);
		MessageBox.this.setAnimationEnabled(true);
		MessageBox.this.center();
		
	}
	
	
}
