package org.mut.ampa.server.manteniment;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;

import org.apache.ibatis.io.Resources;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.mut.ampa.client.screens.manteniment.MantenimentService;
import org.mut.ampa.server.dao.AlumneGermaMapper;
import org.mut.ampa.server.dao.AlumneMapper;
import org.mut.ampa.server.dao.AlumneTutorMapper;
import org.mut.ampa.server.dao.AssociacioMapper;
import org.mut.ampa.server.dao.CursGrupMapper;
import org.mut.ampa.server.dao.CursMapper;
import org.mut.ampa.server.dao.GrupMapper;
import org.mut.ampa.server.dao.TutorMapper;
import org.mut.ampa.server.dao.UsuariMapper;
import org.mut.ampa.shared.LoginCredential;
import org.mut.ampa.shared.exceptions.ServerSideException;
import org.mut.ampa.shared.exceptions.UnAuthorizedException;
import org.mut.ampa.shared.filter.Filter;
import org.mut.ampa.shared.filter.FilterField;
import org.mut.ampa.shared.model.Alumne;
import org.mut.ampa.shared.model.AlumneBrief;
import org.mut.ampa.shared.model.Associacio;
import org.mut.ampa.shared.model.Curs;
import org.mut.ampa.shared.model.CursGrup;
import org.mut.ampa.shared.model.CursGrupInfo;
import org.mut.ampa.shared.model.Grup;
import org.mut.ampa.shared.model.Tutor;
import org.mut.ampa.shared.model.Usuari;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;


/**
 * Implementacio del Servei principal de manteniment de l'aplicació web.
 * @author Mario Unio UOC 2012
 *
 */
public class MantenimentServiceImpl extends  RemoteServiceServlet implements MantenimentService{

	
	private static final long serialVersionUID = -3474043313479687636L;

	private static final String UNAUTHORIZED = "Credencials Invalides"; 
	
	

	//Map que conte les credencials dels usuaris registrats.
	//Key=Nom usuari 
	private Map<String,LoginCredential> logedCredentials = new HashMap<String, LoginCredential>(); 
	
	
	private static SqlSessionFactory sqlSessionFactory;
	
	public MantenimentServiceImpl() {
		try {
			
			//Obtenim una sessio SQL
			String resource = "org/mut/ampa/server/mybatis-config.xml";
			InputStream	inputStream = Resources.getResourceAsStream(resource);
			sqlSessionFactory = new SqlSessionFactoryBuilder().build(inputStream);
			
			
		} catch (IOException e) {
				e.printStackTrace();
		}
	}
	
	
	@Override
	public void registerCredential(LoginCredential credential) 
	{
		logedCredentials.put(credential.getNom(), credential);
	}
	
//====== RPC Entitat Usuari =========

	@Override
	public List<Usuari> getUsuaris(LoginCredential credential, int offset,int maxElements) throws UnAuthorizedException, ServerSideException
	{
		List<Usuari> results =null;
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential)){
				throw new UnAuthorizedException(UNAUTHORIZED);
			}	
			
			//recuperar	
			  UsuariMapper mapper = session.getMapper(UsuariMapper.class);
			 results = mapper.selectAll();
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		return results;
	
	}
	
	@Override
	public Usuari insertUsuari(LoginCredential credential, Usuari user) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential))	throw new UnAuthorizedException(UNAUTHORIZED);
			
			//insert
			 UsuariMapper mapper = session.getMapper(UsuariMapper.class);
			 mapper.insertSelective(user);
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		 return user;
	}

	@Override
	public Usuari updateUsuari(LoginCredential credential, Usuari user) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential))	throw new UnAuthorizedException(UNAUTHORIZED);
			
			//updatar
			 UsuariMapper mapper = session.getMapper(UsuariMapper.class);
			 mapper.updateByPrimaryKey(user);
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		 return user;
	}
	
	@Override
	public List<Usuari> deleteUsuari(LoginCredential credential,Integer id) throws UnAuthorizedException, ServerSideException 
	{
		 List<Usuari> results;
		 
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential))	throw new UnAuthorizedException(UNAUTHORIZED);
			
			//esborrar
			UsuariMapper mapper = session.getMapper(UsuariMapper.class);
			mapper.deleteByPrimaryKey(id);
			results = mapper.selectAll();
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		return results;
	}

	@Override
	public List<Usuari> findUsuari(LoginCredential credential, Filter filter) throws UnAuthorizedException, ServerSideException 
	{
		List<Usuari> results;
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential))	throw new UnAuthorizedException(UNAUTHORIZED);
			
			//recuperar	
			  UsuariMapper mapper = session.getMapper(UsuariMapper.class);
			  
			  FilterField ff[] = filter.getFields();
			  
			  String nomUsuari;
			  if( ff[0].getValue().equalsIgnoreCase("")) nomUsuari="%";
			  else nomUsuari="%"+ff[0].getValue()+"%";
			  
			  String nom;
			  if( ff[1].getValue().equalsIgnoreCase("")) nom="%";
			  else nom="%"+ff[1].getValue()+"%";
			  
			  String cognom1;
			  if( ff[2].getValue().equalsIgnoreCase("")) cognom1="%";
			  else cognom1="%"+ff[2].getValue()+"%";
			  
			  String cognom2;
			  if(ff[3].getValue().equalsIgnoreCase("")) cognom2="%";
			  else cognom2="%"+ff[3].getValue()+"%";
			  
			  results = mapper.selectSearch(nomUsuari, nom, cognom1, cognom2);
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
				
		return results;
	}

//====== RPC Entitat ASSOCIACIO =========
	@Override
	public Associacio updateAssociacio(LoginCredential credencial,Associacio assoc) throws UnAuthorizedException, ServerSideException {

		SqlSession session = sqlSessionFactory.openSession(true);
		
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
			//updatar
			 AssociacioMapper mapper = session.getMapper(AssociacioMapper.class);
			 mapper.updateByPrimaryKey(assoc);
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		 return assoc;
		
	}
	@Override
	public Associacio findAssociacio(LoginCredential credencial, int key) throws UnAuthorizedException, ServerSideException 
	{
		 Associacio result;
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
			
			//recuperar	
			  AssociacioMapper mapper = session.getMapper(AssociacioMapper.class);
			  result = mapper.selectByPrimaryKey(key);
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		return result;
	}
	@Override
	public Associacio insertAssociacio(LoginCredential credencial,Associacio assoc) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
		
			//updatar
			 AssociacioMapper mapper = session.getMapper(AssociacioMapper.class);
			 mapper.insertSelective(assoc);
			 
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		return assoc;
	}

//====== RPC Entitat Curs =========
	@Override
	public List<Curs> getCursos(LoginCredential credential, int i, int j)throws UnAuthorizedException, ServerSideException 
	{
		List<Curs> results;
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential))
				throw new UnAuthorizedException(UNAUTHORIZED);
			
			
			//recuperar	
			  CursMapper mapper = session.getMapper(CursMapper.class);
			  results = mapper.selectAll();
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		return results;
	}
	@Override
	public Curs insertCurs(LoginCredential credencial, Curs curs) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
			
			//insert
			CursMapper mapper = session.getMapper(CursMapper.class);
			mapper.insertSelective(curs);
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
	
				
		return curs;
	}
	@Override
	public Curs updateCurs(LoginCredential credencial, Curs curs) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
		
			//updatar
			CursMapper mapper = session.getMapper(CursMapper.class);
			 mapper.updateByPrimaryKey(curs);
			
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		 return curs;
		
	}
	@Override
	public List<Curs> deleteCurs(LoginCredential credential, Integer id) throws UnAuthorizedException, ServerSideException 
	{
		List<Curs> results;
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential)) throw new UnAuthorizedException(UNAUTHORIZED);
			
			//esborrar
			CursMapper mapper = session.getMapper(CursMapper.class);
			mapper.deleteByPrimaryKey(id);
			results = mapper.selectAll();
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
		
		 
		return results;
	}
	@Override
	public List<Curs> findCurs(LoginCredential credential, Filter filter) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential))
				throw new UnAuthorizedException(UNAUTHORIZED);
			
			// recuperar
			CursMapper mapper = session.getMapper(CursMapper.class);

			FilterField ff[] = filter.getFields();

			String descripcio;
			if (ff[0].getValue().equalsIgnoreCase(""))
				descripcio = "%";
			else
				descripcio = "%" + ff[0].getValue() + "%";

			List<Curs> results = mapper.selectSearch(descripcio);

			return results;
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}

		
	}

//====== RPC Entitat Grups =========
	
	@Override
	public List<CursGrupInfo> getGrups(LoginCredential credential, int i, int j) throws UnAuthorizedException, ServerSideException
	{
		List<CursGrupInfo> ret = new ArrayList<CursGrupInfo>();
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential))
				throw new UnAuthorizedException(UNAUTHORIZED);
		
			
			
			//recuperar	
		  GrupMapper mapper = session.getMapper(GrupMapper.class);
		  CursMapper cMapper = session.getMapper(CursMapper.class);
		  
		  List<Grup> results = mapper.selectAll();
		  for (Grup grup : results)
		  {
			 Curs c= cMapper.selectByPrimaryKey(grup.getIdCurs());
			 
			 CursGrupInfo tmp = new CursGrupInfo();
			 tmp.setIdGrup(grup.getId());
			 tmp.setDescripcioGrup(grup.getDescripcio());
			 tmp.setIdCurs(grup.getIdCurs());
			 tmp.setDescripcioCurs(c.getDescripcio());
			 
			 ret.add(tmp);
		  }
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
		
	  
			return ret;
	}
	@Override
	public CursGrupInfo insertGrup(LoginCredential credencial, CursGrupInfo grup) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
					
			//insert
			GrupMapper mapper = session.getMapper(GrupMapper.class);
			
			Grup tmp = new Grup();
			tmp.setId(grup.getIdGrup());
			tmp.setDescripcio(grup.getDescripcioGrup());
			tmp.setIdCurs(grup.getIdCurs());
			mapper.insertSelective(tmp);
					
			return grup;

			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
	}
	@Override
	public CursGrupInfo updateGrup(LoginCredential credencial, CursGrupInfo grup) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
			
			//updatar
			GrupMapper mapper = session.getMapper(GrupMapper.class);
			Grup tmp = new Grup();
			tmp.setId(grup.getIdGrup());
			tmp.setDescripcio(grup.getDescripcioGrup());
			tmp.setIdCurs(grup.getIdCurs());
			
			mapper.updateByPrimaryKeySelective(tmp);
			
			return grup;
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
	}
	@Override
	public List<CursGrupInfo> deleteGrup(LoginCredential credential,int idGrup,int idCurs) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential)) throw new UnAuthorizedException(UNAUTHORIZED);
			
			//esborrar
			GrupMapper mapper = session.getMapper(GrupMapper.class);
			mapper.deleteByPrimaryKey(idGrup,idCurs);
			
			//recuperar	
			  CursMapper cMapper = session.getMapper(CursMapper.class);
			  
			  List<Grup> results = mapper.selectAll();
			  List<CursGrupInfo> ret = new ArrayList<CursGrupInfo>();
			  
			  for (Grup grup : results)
			  {
				 Curs c= cMapper.selectByPrimaryKey(grup.getIdCurs());
				 
				 CursGrupInfo tmp = new CursGrupInfo();
				 tmp.setIdGrup(grup.getId());
				 tmp.setDescripcioGrup(grup.getDescripcio());
				 tmp.setIdCurs(grup.getIdCurs());
				 tmp.setDescripcioCurs(c.getDescripcio());
				 
				 ret.add(tmp);
			  }
			  
				return ret;
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
		
	}
	@Override
	public List<CursGrupInfo> findGrup(LoginCredential credential,Filter filter) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			// validar credencials
			if (!isValidCredential(credential))
				throw new UnAuthorizedException(UNAUTHORIZED);

			// recuperar
			GrupMapper mapper = session.getMapper(GrupMapper.class);

			FilterField ff[] = filter.getFields();

			String descripcioGrup;
			if (ff[0].getValue().equalsIgnoreCase("")) descripcioGrup = "%";
			else descripcioGrup = "%" + ff[0].getValue() + "%";

			String descripcioCurs;
			if (ff[1].getValue().equalsIgnoreCase("")) descripcioCurs = "%";
			else descripcioCurs = "%" + ff[1].getValue() + "%";
			
			List<Grup> results = mapper.selectSearch(descripcioGrup,descripcioCurs);
			
			CursMapper cMapper = session.getMapper(CursMapper.class);
			
			List<CursGrupInfo> ret = new ArrayList<CursGrupInfo>();
			  
			  for (Grup grup : results)
			  {
				 Curs c= cMapper.selectByPrimaryKey(grup.getIdCurs());
				 
				 CursGrupInfo tmp = new CursGrupInfo();
				 tmp.setIdGrup(grup.getId());
				 tmp.setDescripcioGrup(grup.getDescripcio());
				 tmp.setIdCurs(grup.getIdCurs());
				 tmp.setDescripcioCurs(c.getDescripcio());
				 
				 ret.add(tmp);
			  }
			  
				return ret;

			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
			}

//====== RPC Entitat Tutor =========
	@Override
	public List<Tutor> getTutors(LoginCredential credential, int i, int j)throws UnAuthorizedException, ServerSideException 
	{

		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential))
			{
			 throw new UnAuthorizedException(UNAUTHORIZED);
			}
				
			//recuperar	
			  TutorMapper mapper = session.getMapper(TutorMapper.class);
			  List<Tutor> results = mapper.selectAll();
				  
			return results;
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
	
	}	
	@Override
	public List<Tutor> deleteTutor(LoginCredential credential, Integer id)throws UnAuthorizedException, ServerSideException {
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential)) throw new UnAuthorizedException(UNAUTHORIZED);
					
			//esborrar
			TutorMapper mapper = session.getMapper(TutorMapper.class);
			mapper.deleteByPrimaryKey(id);
			List<Tutor> results = mapper.selectAll();
					 
			return results;
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
			}
	@Override
	public List<Tutor> findTutors(LoginCredential credential,Filter filter) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential)) throw new UnAuthorizedException(UNAUTHORIZED);
							
			//recuperar	
			  TutorMapper mapper = session.getMapper(TutorMapper.class);
					  
			  FilterField ff[] = filter.getFields();
					  
			  String nom;
			  if( ff[0].getValue().equalsIgnoreCase("")) nom="%";
			  else nom="%"+ff[0].getValue()+"%";
					  
			  String cognom1;
			  if( ff[1].getValue().equalsIgnoreCase("")) cognom1="%";
			  else cognom1="%"+ff[1].getValue()+"%";
					  
			  String cognom2;
			  if(ff[2].getValue().equalsIgnoreCase("")) cognom2="%";
			  else cognom2="%"+ff[2].getValue()+"%";
					  
			  List<Tutor> results = mapper.selectSearch( nom, cognom1, cognom2);
			return results;
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
	
		
		}
	@Override
	public Tutor insertTutor(LoginCredential credencial, Tutor tutor) throws UnAuthorizedException, ServerSideException {
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
					
			//insert
			TutorMapper mapper = session.getMapper(TutorMapper.class);
			 mapper.insertSelective(tutor);
					
			 return tutor;
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
		
			}
	@Override
	public Tutor updateTutor(LoginCredential credencial, Tutor tutor) throws UnAuthorizedException, ServerSideException 
	{
		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
		
			//updatar
			 TutorMapper mapper = session.getMapper(TutorMapper.class);
			 mapper.updateByPrimaryKey(tutor);
			 return tutor;
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
	}

//====== RPC Entitat Alumne =========
	@Override
	public List<AlumneBrief> getAlumnes(LoginCredential credential, int i, int j) throws UnAuthorizedException, ServerSideException 
	{

		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential)) throw new UnAuthorizedException(UNAUTHORIZED);
					
			return getBrief();
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
	}
	@Override
	public List<AlumneBrief> deleteAlumne(LoginCredential credential, Integer id) throws UnAuthorizedException, ServerSideException {

		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credential)) throw new UnAuthorizedException(UNAUTHORIZED);
					
			//esborrar
			AlumneMapper mapper = session.getMapper(AlumneMapper.class);
			mapper.deleteByPrimaryKey(id);
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
				 
		return getBrief();
	}


	@Override
	public List<AlumneBrief> findAlumne(LoginCredential credential,
			Filter filterValues) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public Alumne getAlumneInfo(LoginCredential credencial,AlumneBrief alumneBrief) throws UnAuthorizedException, ServerSideException 
	{

		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//validar credencials
			if(!isValidCredential(credencial)) throw new UnAuthorizedException(UNAUTHORIZED);
			
			 AlumneMapper mapperAlumne = session.getMapper(AlumneMapper.class);
			 Alumne result = mapperAlumne.selectByPrimaryKey(alumneBrief.getId());
					
			return result;
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
	
		
		
		
	}


	@Override
	public List<AlumneBrief> getGermansInfo(LoginCredential credencial,
			AlumneBrief alumneBrief) throws UnAuthorizedException {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public CursGrupInfo getGrupsCurs(LoginCredential credencial)throws UnAuthorizedException {
		CursGrupInfo ret = new CursGrupInfo();
		ret.setIdCurs(1);
		ret.setDescripcioCurs("CURS 2012-2013");
		ret.setDescripcioGrup("P3-A");
		ret.setIdGrup(13);
		return ret;
	}


	@Override
	public AlumneBrief insertAlumne(LoginCredential credencial, Alumne alumne)
			throws UnAuthorizedException {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public AlumneBrief updateAlumne(LoginCredential credencial, Alumne alumne)
			throws UnAuthorizedException {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public List<Tutor> getTutorsInfo(LoginCredential credencial,
			AlumneBrief alumneBrief) throws UnAuthorizedException {
		// TODO Auto-generated method stub
		return null;
	}

	
	private List<AlumneBrief> getBrief() throws ServerSideException {
		List<AlumneBrief> result = new ArrayList<AlumneBrief>();
		

		SqlSession session = sqlSessionFactory.openSession(true);
		try{
			//recuperar	
			  AlumneMapper mapperAlumne = session.getMapper(AlumneMapper.class);
			  List<Alumne> results = mapperAlumne.selectAll();
			for (Alumne alumne : results) 
			{
			
				CursGrupMapper cursGrupMapper = session.getMapper(CursGrupMapper.class);	
				CursGrup cg = cursGrupMapper.selectByPrimaryKey(alumne.getId());
			
				GrupMapper grupMapper = session.getMapper(GrupMapper.class);
				Grup grup = grupMapper.selectByPrimaryKey(cg.getGRUP_id(), cg.getGRUP_idCurs());
				
				AlumneBrief tmp = new AlumneBrief();
				tmp.setId(alumne.getId());
				tmp.setNom(alumne.getNom());
				tmp.setCognom1(alumne.getCognom1());
				tmp.setCognom2(alumne.getCognom2());
				tmp.setCursDescripcio(grup.getDescripcio());
				
				AlumneTutorMapper alumTut = session.getMapper(AlumneTutorMapper.class);
				List<Integer> idsTutor =alumTut.selectTutors(alumne.getId());
				tmp.setIdsTutor(idsTutor);
				
				AlumneGermaMapper alumGer = session.getMapper(AlumneGermaMapper.class);
				List<Integer> idsAlumneGerma =alumGer.selectGermans(alumne.getId());
				tmp.setIdsAlumneGerma(idsAlumneGerma);
				
				result.add(tmp);
			
			}	  
			  
			return result;
		
			  
			
		}catch(Exception e){
			throw new ServerSideException(e.getMessage());
		}finally{
			session.close();	
		}
		
		
		}
	
	
	
	
	/* =*=*=*=*=*=*=*=*=* RPC REPORTS *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*== */
		
	@Override
	public String getPdfReport(LoginCredential credencial,String fileName, HashMap <String,Object> param) throws ServerSideException, UnAuthorizedException
		{
		//validar credencials
				if(!isValidCredential(credencial))
					{
					 throw new UnAuthorizedException(UNAUTHORIZED);
					}
		
			try
				{
				Class.forName("com.mysql.jdbc.Driver");
				Connection con = DriverManager.getConnection ("jdbc:mysql://localhost:3306/AMPAROSELLA", "root", "12root34");
				String filePath = getServletConfig().getServletContext(). getRealPath(fileName);
				JasperPrint print = JasperFillManager.fillReport(new FileInputStream(filePath+".jasper"), param, con) ;
				String newFileName = filePath+".pdf";
				JasperExportManager.exportReportToPdfFile(print, newFileName);
				return fileName+".pdf";
				}
				catch (SQLException ex)
					{
					throw new ServerSideException(ex.getMessage());
					}
					catch (ClassNotFoundException ex)
					{
						System.out.println("getPDF: "+ex.getMessage());
						throw new ServerSideException(ex.getMessage());
					}
					catch (JRException ex)
					{
						System.out.println("getPDF: "+ex.getMessage());
						throw new ServerSideException(ex.getMessage());
					} catch (FileNotFoundException e) {
						System.out.println("getPDF: "+e.getMessage());
						throw new ServerSideException(e.getMessage());
					}
		}
	
	/* =*=*=*=*=*=*=*=*=* METODES INTERNS *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*== */
	
	private boolean isValidCredential(LoginCredential credential)
	{
		return logedCredentials.containsKey(credential.getNom());
		
	}

}
